"""Module containing SOI-module 'Subtraktorkoder'."""
import string
from PySide2.QtWidgets import QTableWidgetItem
from PySide2 import QtGui
from PySide2.QtCore import Qt
from soitool.modules.module_base import resize_table
from soitool.modules.code_table_base import CodeTableBase

START_NO_OF_CODES = 7  # Maximum value is len(ALPHABET)/2 columns = 13
CODE_LENGTH = 8
SPACE_INTERVAL = 4
SPACE_AMOUNT = 2
ALPHABET = string.ascii_uppercase  # Characters for first and second column

HEADLINE_TEXT = "Subtraktorkoder"


class SubtractorcodesModule(CodeTableBase):
    """..."""

    def __init__(self, size=None, data=None):
        self.type = "SubtractorcodesModule"

        if START_NO_OF_CODES > 13:
            raise ValueError(
                "Invalid value for CONSTANT 'START_NO_OF_CODES': "
                "'{}'".format(START_NO_OF_CODES)
            )

        self.start_no_of_codes = START_NO_OF_CODES
        self.code_length = CODE_LENGTH
        self.space_interval = SPACE_INTERVAL
        self.space_amount = SPACE_AMOUNT
        self.code_character_type = "digits"
        self.headline = HEADLINE_TEXT

        CodeTableBase.__init__(self, size, data)
        # If headline changes
        self.cellChanged.connect(
            lambda: resize_table(self, resize_column=False)
        )

    def insert_row_identifiers(self):
        """Insert table data.

        Parameters
        ----------
        data : List
            List of codes to insert.
        """
        split_one = ALPHABET[0 : self.rowCount()]
        split_two = ALPHABET[self.rowCount() : self.rowCount() * 2]

        for i in range(2):
            for j in range(self.rowCount()):
                text = split_one[j] if i == 0 else split_two[j]
                item = QTableWidgetItem(text)
                item.setFlags(item.flags() ^ Qt.ItemIsEditable)
                self.setItem(j, i, item)

    def add_row(self):
        """..."""
        row_index = self.currentRow()
        # If maximum amount of rows not reached and a row is selected
        # (+ 1 to skip row containing headline)
        if self.rowCount() < len(ALPHABET) / 2 + 1 and row_index != -1:
            # Generate unique code and insert row
            code = self.generate_unique_authentication_code()
            self.insertRow(row_index + 1)

            # Insert values in first and second column
            split_one = ALPHABET[0 : self.rowCount() - 1]
            split_two = ALPHABET[self.rowCount() - 1 : self.rowCount() * 2 - 2]

            for i in range(2):
                for j in range(1, self.rowCount()):  # From 1 to skip headline
                    text = split_one[j - 1] if i == 0 else split_two[j - 1]
                    item = QTableWidgetItem(text)
                    item.setFlags(item.flags() ^ Qt.ItemIsEditable)
                    self.setItem(j, i, item)

            # Insert code in third column
            item_third = QTableWidgetItem(code)
            item_third.setFlags(item_third.flags() ^ Qt.ItemIsEditable)
            self.setItem(row_index + 1, 2, item_third)

            # Resize code-column in case it got wider
            # Example: 'BGD' is wider than 'III' (depending on font)
            self.resizeColumnToContents(2)
            resize_table(self, resize_column=False)

    def remove_row(self):
        """Remove selected row."""
        row_index = self.currentRow()
        # If at least one row (+ headline-row) exists and a row other than
        # headline-row is selected
        if self.rowCount() > 2 and row_index != 0 and row_index != -1:
            # Remove row
            self.removeRow(row_index)

        split_one = ALPHABET[0 : self.rowCount() - 1]
        split_two = ALPHABET[self.rowCount() - 1 : self.rowCount() * 2 - 2]
        for i in range(2):
            for j in range(1, self.rowCount()):  # From 1 to skip headline
                # Insert values in first and second column
                text = split_one[j - 1] if i == 0 else split_two[j - 1]
                self.item(j, i).setText(text)

    @staticmethod
    def get_user_friendly_name():
        """Get user-friendly name of module."""
        return "Subtraktorkoder"

    @staticmethod
    def get_icon():
        """Get icon of module."""
        return QtGui.QIcon("soitool/media/authenticationboardmodule.png")
